using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Drawing.Imaging;
using System.IO;
using System.Net;
using System.Threading;

namespace WindowsApplication1
{
	public partial class Form1 : Form
	{

		byte[] send = { 0xaa, 0, 0, 0, 0, 0 };// JPEG camera command header
		byte[] rec = new byte[6];// JPEG camera command reply buffer

        int[] BaudTable = { 460800, 230400, 115200, 57600, 28800, 14400};// Baud rate table
        byte[] CamData = { 2, 3, 4, 5, 6, 7};// Baud rate table
        int syncseq = 0;		// SYNC counter
		int sizeindex = 0;	    // JPEG resolution 
		int freq = 0;			// Light frequency
		int baudindex = 0;	    // Baud rate combobox index
		int boxsize = 0;		// PictureBox size
		int packetsize = 512;   // JPEG data block size
		int saveseq = 0;		// Save filename seq number
		int playseq = 0;		// Load filename seq number
		int margine = 0;		// Form width

		volatile int cmd = 0;	// Action command
		volatile int timercount = 0;// Timer counter for interval

		string port = "";		// COM port name
		string filename = "";	// Filename

		bool active = false;// Thread working flag
		bool abort = false;	// Abort flag
		bool quit = false;	// Closing flag
		bool debug = false;	// Debug flag

		MemoryStream ms;		// Memory stream for JPEG cache
		Point picpos;			// Picture box position

		//
		//Form1 startup
		//
		public Form1()// Form class
		{
			InitializeComponent();		// Init event handler
			CheckForIllegalCrossThreadCalls = false;// Cross thread check disable
			margine = this.Width - pictureBox1.Width;// Get picture box initial width margin
			picpos = pictureBox1.Location;	// Get picture box position
			string[] ports = System.IO.Ports.SerialPort.GetPortNames();// Get available COM port list
			foreach (string pn in ports)// Repeat for all COM port list
				comboBox1.Items.Insert(0, pn);// Add combo box items 
			comboBox1.SelectedIndex = 0;// Set baud rate combo box index to top
			port = (string)comboBox1.SelectedItem;// Get port name from top of items

			label3.Text = "";	// Clear label3 (Main text)
			label7.Text = "";	// Clear label7 (timer text)
			label8.Text = "";	// Clear label8 (File index text)
			
			comboBox2.Items.Add("80x64");	// Add resolution combo box item
			comboBox2.Items.Add("160x120");	// Add resolution combo box item
			comboBox2.Items.Add("320x240");	// Add resolution combo box item
			comboBox2.Items.Add("640x480");	// Add resolution combo box item
            comboBox2.SelectedIndex = 2;	// Set resolution combo box select index

            comboBox3.Items.Add("460800");	// Add speed combo box item
            comboBox3.Items.Add("230400");	// Add speed combo box item
            comboBox3.Items.Add("115200");	// Add speed combo box item
			comboBox3.Items.Add("57600");	// Add speed combo box item
			comboBox3.Items.Add("28800");	// Add speed combo box item
			comboBox3.Items.Add("14400");	// Add speed combo box item
			comboBox3.SelectedIndex = 2;	// Set speed combo box iselect index

			comboBox4.Items.Add("50Hz");	// Add frequency combo box item
			comboBox4.Items.Add("60Hz");	// Add frequency combo box item
			comboBox4.SelectedIndex = 0;	// Set frequency combo box select index

            comboBox5.Items.Add("_CNg");	// Add frequency combo box item
            comboBox5.Items.Add("CAX");	// Add frequency combo box item
            comboBox5.Items.Add("lbg[N");	// Add frequency combo box item
            comboBox5.SelectedIndex = 0;	// Set frequency combo box select index

            textBox1.Text = "15";	// Set interval second box default value

			backgroundWorker1.RunWorkerAsync();// Start action thread
		}

		//
		// Form closing event handler
		//
		private void Form1_FormClosing(object sender, FormClosingEventArgs e)
		{
			abort = true;	// Set abort flag
			quit = true;	// Set quit flag
			while (active)// Wait while action thread is active
			{
				Application.DoEvents();// Let allow another process run
				System.Threading.Thread.Sleep(1);// Allow action thread work
			}
			if (serialPort1.IsOpen)// Serial port is opened?
			{
				try
				{
					serialPort1.Close();// Try to close serial port
				}
				catch (Exception ex)	// Catch error
				{
					MessageBox.Show(ex.Message, "VAʐM");// Display serial execption text
				}
			}
		}

		//
		// Form size changed event handler
		//
		private void Form1_SizeChanged(object sender, EventArgs e)
		{
			if (boxsize != 4)
			{
				pictureBox1.Width = this.Width - margine;// Get new picture box width
				pictureBox1.Height = pictureBox1.Width * 3 / 4;// Set new picture box hight
			}
			else
			{
				pictureBox1.Width = this.ClientSize.Width;// Set maximum X size
				pictureBox1.Height = this.ClientSize.Width * 3 / 4;// Set maximum Y size
			}
		}

		//
		// Error message print
		//
		private void MessageOut(string str)
		{
			label3.Text = str;  // Set label text
			label3.Update();	// Update label control
		}

		//
		//	Send command to camera
		//
		private bool SendCmd(byte cmd, int p1, int p2)
		{
			send[1] = cmd;// Set command
			send[2] = (byte)(p1 & 255);	// Set 3rd parameter
			send[3] = (byte)(p1 >> 8);	// Set 4th parameter
			send[4] = (byte)(p2 & 255);	// Set 5th parametar
			send[5] = (byte)(p2 >> 8);	// Set 6th parameter
			try
			{
				serialPort1.Write(send, 0, 6);// Send 6 byte
				return false;	// Normal return
			}
			catch
			{
				return true;	// Port error return
			}
		}

		//
		//	Receive JPEG camera reply
		//
		private int RecData(int offset, int size)
		{
			int i;	// Read number
			try
			{
				i = serialPort1.Read(rec, offset, size);// Try read data from serial port
			}
			catch (TimeoutException)// Timeout happens
			{
				i = 0;	// Clear read counter
			}
			catch (InvalidOperationException)// Port error
			{
				i = -1;	// Error return
			}
			return i;	// Return with read counter
		}

		//
		// Flush serial buffer
		//
		private bool Flush()
		{
			int i;		// Return value
			for (; ; )// Repeat forever
			{
				i = RecData(0, 1);// Read 1 byte
				if (i < 0)				// Port error happens
					return true;		// Error return
				if (i == 0)				// No more data?
					return false;		// Normal return
			}
		}

		//
		// Wait JPEG camera response
		//
		private int WaitRes()
		{
			int i, j, k;		// Temp works
			for (j = 0; j < 6; j++)// Try to read 6 bytes
			{
				for (i = 0; i < 2000; i++)// Try 2 seconds
				{
					k = RecData(j, 1);// Try to read 1 byte
					if (k < 0)	// Port error happens?
						return -1;// Port error return
					if (k > 0)	// If some data arrived?
						break;		// Exit 2 second trial loop
					System.Threading.Thread.Sleep(1);// Wait 1ms and allow another thread works
				}
				if (i == 2000)// Time out?
				{
					MessageOut("^CAEgł");// Display error text
					return 1;		// Timeout return
				}
			}
			if (rec[0] != 0xaa)// Check replay packet header is 0xaa
			{
				MessageOut("J̉sł");// Display invalid packet
				return 2;			// Invalid data return
			}
			return 0;				// Normal return
		}

		//
		//	Check JPEG camera response		
		//
		private int CheckRes()
		{
			int i;				// Return value work
			i = WaitRes();// Wait camera response
			if (i < 0)		// Port error happens?
				return -1;	// Port error return
			if (i > 0)		// Time out or invalid reply happens?
				return i;		// Time out return
			if (rec[1] != 0x0e)// Check 2nd byte is ACK?
			{
				MessageOut("J̉ACKłȂ("+rec[1]+","+rec[4]+")");//Disp error text and error code
				return 2;		// Invalid data return
			}
			return 0;			// Normal return
		}

		//
		// Destroy MemoryStream
		//
		private void DisposeMs()
		{
			try
			{
				ms.Dispose();// Try memory stream dispose
			}
			catch		    // Error handling
			{				// Do nothing when error happens
			}
		}

		//
		//	Delay miliseconds
		//
		private void Delay(int t)
		{
				System.Threading.Thread.Sleep(t);// System call delay
		}

		//
		//	RS-232C port open
		//
		private bool RsOpen()
		{
			if (!serialPort1.IsOpen)// Not oen yet?
			{
				serialPort1.PortName = port;	// Set port name
				try
				{
					serialPort1.Open();	// Try to open serial port
					serialPort1.RtsEnable = true;// Enable RTS for flow control
                    serialPort1.BaudRate = BaudTable[baudindex];// Set baud rate
                }
				catch (Exception ex)	// Open error happens?
				{
					MessageOut(ex.Message);// Display error message
					return true;				// Error return
				}
			}
			return false;	// Normal return
		}

        //
        //	Set baud rate
        //
        private bool SetBaud(int i)
        {
            try
            {
                serialPort1.BaudRate = BaudTable[i];// Set baud rate
            }
            catch       // Open error happens?
            {
                return false;				// Error return
            }
            return true;	// Normal return
        }

        //
        //  Sync sub
        //
        private int CamSyncSub(byte m)
        {
            int i, j;
            if (Flush())	// Flush serial buffer first
                return -1;	// Port error return
            for (i = 0; i < m; i++)// Try specified times
            {
                if (SendCmd(0x0d, 0, 0))// Send SYNC command
                    return -1;	// Port error return
                if (comboBox5.SelectedIndex != 0)   // Wirless serial mode?
                    Delay(50);	// Wait 50ms for two way cominucation
                else
                    Delay(5 + baudindex * 5);// Wait 5ms + baud rate delay
                j = RecData(0, 1);// Sense 1 byte
                if (j < 0)		// Port error happens?
                    return -1;	// Port error return
                if (j == 0)		// No data
                    continue;		// Try again
                j = RecData(1, 5);	// Read rest of data
                if (j < 0)		// Port error
                    return -1;	// Port error return
                if (j == 5 && rec[0] == 0xaa && rec[1] == 0x0e)// Reply is right ACK packet?
                    break;			// ACK has come then exit loop
                if (Flush())	// Flush invalid data
                    return -1;	// Port error return
            }
            syncseq++;			// Increment SYNC seq number for debug
            if (debug)				// Debug flag set?
                label7.Text = i.ToString() + "," + syncseq.ToString();// Display SYNC count
            if (i == m)		// Try specified times?
                return 1;			// Timeout return
            j = WaitRes();	// Wait JPEG camera's SYNC packet
            if (j != 0)			// Some error happens?
                return j;			// Error return
            if (rec[1] != 0x0d)// Check SYNC packet
            {
                MessageOut("JSYNCvُł");
                return 2;			// Error return
            }
            if (SendCmd(0x0e, 0x0d, 0))// Send ACK replay
                return -1;		// Port error return
            Delay(50);// Hot start then wait while to finish ACK pakcet process
            return 0;
        }

		//
		//JPEG camera SYNC
		//
		private int CamSync()
		{
			int i;			// Temp work
			if (checkBox2.Enabled && checkBox2.Checked)	// Wireless mode?
			{
				serialPort1.RtsEnable = true;// Set RTS on for light control
				label3.Text = "Ɩnmɂ܂";// Display light on
				progressBar1.Value = 0;// Clear progress bar
				Delay(1000);				// Wait 1 second for light become stable
			}
            if (checkBox5.Checked && comboBox5.SelectedIndex == 0)   // C1098 mode & direct connection?
            {
                if (!SetBaud(baudindex))  // Try to set current speed
                    return 1;   // Port dose not accept cuurent speed
                i = CamSyncSub(2);   // Try to sync in current speed
                if (i != 0) // Fail sync?
                {
                    for (i = 0; i < 5; i++) // Scan camera internal serial speed
                    {
                        if (SetBaud(i)) // Try set baud rate in the table
                            if (CamSyncSub(2) == 0) // If sync suceed then exit loop
                                break;
                    }
                    if (i == 5)  // Sync speed found?                    
                    {   // it means cold sync(use 14400bps for sync)
                        try
                        {
                            serialPort1.BaudRate = 14400;// Set baud rate 14400
                            i = CamSyncSub(70);   // Try cold sync
                        }
                        catch
                        {
                            i = -1; // Fail cold sync
                        }
                    }
                    else
                        i = 0;  // Hot sync sucseeded then clear error flag
                }
            }
            else  // C328 mode or wireless mode
                i = CamSyncSub(70);   // Try to sync
            if (i != 0)  // Some error happens?
                return i;   // Error return
			if(checkBox3.Checked && !checkBox5.Checked)	// Warming up mode(only for C328)?
			{			// Cold start then wait 3 second for AGC stable(C328 only)
				label3.Text = "J̊xݒ蒆";// Display camera is now warming up
				progressBar1.Value = 0;// Set progtress bar 0
				Delay(1000);					// Wait 1 sec
				progressBar1.Value = 33;// Ser progress bar 33%
				Delay(1000);					// Wait 1 sec
				progressBar1.Value = 66;// Set progress bar 66%
				Delay(1000);					// Wait 1 sec
				progressBar1.Value = 100;// Set progress bar 100%
				label3.Text = "";			// Clear displayed text
			}
			return 0;								// Normal return
		}

		//
		//	Setup camera parameters
		//
		private int CamSetup()
		{
			int i;							// Return value work
			if (comboBox5.SelectedIndex != 0)// Wireless mode set?
				packetsize = 256;	// Set JPEG packet size 256 (SS-1 buffer size)
			else
				packetsize = 512;	// Set JPEG packet size 512 (C328 maximum setteing)
			switch (sizeindex)	// Switch by resolution
			{
			case 0:
                if (checkBox5.Checked)
                {
                    if (SendCmd(0x01, 0x0700 | CamData[baudindex], 0x0100))// JPEG mode 80x64
                        return -1;	// Port error return
                }
                else
                {
                    if (SendCmd(0x01, 0x0700, 0x0103))// JPEG mode 80x64
                        return -1;	// Port error return
                }
				break;
			case 1:
                if (checkBox5.Checked)
                {
                    if (SendCmd(0x01, 0x0700 | CamData[baudindex], 0x0300))// JPEG mode 160x120
                        return -1;	// Port error return
                }
                else
                {
    				if (SendCmd(0x01, 0x0700, 0x0303))// JPEG mode 160x120
	    				return -1;	// Port error return
				}
                break;
			case 2:
                if (checkBox5.Checked)
                {
                    if (SendCmd(0x01, 0x0700 | CamData[baudindex], 0x0500))// JPEG mode 320x240
                        return -1;	// Port error return
                }
                else
                {
                    if (SendCmd(0x01, 0x0700, 0x0503))// JPEG mode 320x240
                        return -1;	// Port error return
                }
                break;
			case 3:
                if (checkBox5.Checked)
                {
                    if (SendCmd(0x01, 0x0700 | CamData[baudindex], 0x0700))// JPEG mode 640x480
                        return -1;	// Port error return
                }
                else
                {
                    if (SendCmd(0x01, 0x0700, 0x0703))// JPEG mode 640x480
                        return -1;	// Port error return
                }
				break;
			}
            if (checkBox5.Checked)
            {
                Delay(20);  // Wait until Init command sent
                try
                {
                    serialPort1.BaudRate = BaudTable[baudindex];// Set baud rate
                }
                catch
                {
                    MessageOut("p[^ԈĂ܂");
                    return -1;			// Error return
                }
            }
            i = CheckRes();	// Check response
			if (i != 0)
				return i;			// Error return
            if (checkBox5.Checked)              // C1098?
                Delay(50);          // Wait 50mS after C1098 init command
            if (SendCmd(0x06, (packetsize << 8) & 0xff00 | 0x08, (packetsize >> 8) & 0xff))// Set packet size
				return -1;		// Port error
			i = CheckRes(); // Wait response
			if (i != 0)			// Some error?
                return i; 		// Error return
            if (!checkBox5.Checked)
            {
                if (SendCmd(0x13, freq, 0))// Set light frequency command
                    return -1;		// Port error
                i = CheckRes();	// Wait response
                if (i != 0)			// Some error?
                    return i; 		// Port error
            }
            return 0;				// Normal return
		}

		//
		//	Get JPEG camera picture data
		//
		private int GetPict()
		{
			int i, j, k, l = 0, s;// Temp work
			int seq;	// Packet seq number
			int size;	// JPEG data size
			byte[] buf = new byte[512];// Picture data recieve buf

			if (RsOpen())	// Try to open COM port
				return -1;	// Serial port error
			i = CamSync();// SYNC Camera
			if (i != 0)		// Some error?
				return i;		// Error return
            i = CamSetup();// Setup resolutions
			if (i != 0)		// Some error happens?
				return i;		// Error return
            if (SendCmd(0x05, 0, 0))// Send snapshot command (compressed)
				return -1;	// Port error
			i = CheckRes();// Check response
			if (i != 0)		// Some error?
				return i;		// Port error
			if (checkBox2.Enabled && checkBox2.Checked)		// Light control mode enable? and light is on?
				serialPort1.RtsEnable = false;// Clear RTS to turn off light
			if (SendCmd(0x04, 0x01, 0))// Send get picture command
				return -1;	// Port error
			i = CheckRes();// Check response
			if (i != 0)		// Some error?
				return i;		// Port error
			i = WaitRes();// Wait response
			if (i != 0)		// Error happens?
				return i;		// Error return
			if (rec[1] != 0x0a)// Check response is 0x0a
			{
				MessageOut("J̉sł");// Display invalid reply
				return 2;		// Data error return
			}
			size = rec[5] * 65536 + rec[4] * 256 + rec[3];// Get JPEG data size
			if (size < 0 || size > 0x40000)// Check JPEG data size
			{
				MessageOut("摜̃TCYsł");// Display invalid JPEG size
				return 2;		// data error return
			}
			DisposeMs();	// Dispose previous memory stream
			ms = new MemoryStream(size);// Create new memory stream
			s = size;			// Copy JPEG data size
			progressBar1.Value = 0;// Set progress bar 0% first
			label3.Text = "ǂݍݒ";// Show reading message
			if (size != 0)// data exists?
			{
				seq = 0;		// Clear seq num first
				while (size != 0)	// Get data loop
				{
					if (abort)// Abort request by user?
						break;	// Exit JPEG data packet get loop
					if (SendCmd(0x0e, 0, seq))// Send JPEG packet request (seq required)
						return -1;		// Port error return
					k = packetsize;	// Set packet size
					if (size + 6 <= k)	// Final packet?
						k = size + 6;	// Reset final packet size
					Delay(k / 8);		// Wait untill packet data arrive
					l = 0;					// Crear read counter first
					for (i = 0; i < k; i++)// Repeat for packet data size 
					{
						for (j = 0; j < 1000; j++)// Wait 1 second for data arrival
						{
							l = RecData(0, 1);// Sense 1 byte
							if (l < 0)	// Port error happens?
								break;		// Error loop exit
							if (l == 1)	// Data has come?
								break;		// Normal exit 1 second wait loop
							Delay(1);		// Wait 1 milisecond and anither thread can work
						}
						if (l < 0)	// Port error happens?
							break;		// Exit packet read loop
						if (j == 1000)// Time out happens?
						{		// l must be 0
							MessageOut("JPEGf[^̃^CAEgł");// Display error text
							break;		// Exit Packet read loop
						}
						buf[i] = rec[0];// Save get data to JPEG data buffer
					}
					if (l < 0 || l == 0)// Port error or time out happens?
						break;			// Exit packet loop
					ms.Write(buf, 4, k - 6);// Write JPEG data to memory stream
					size -= k - 6;// Sub read length
					seq++;				// Advance packet seq number
					progressBar1.Value = (s - size) * 100 / s;// Show progress bar
				}
				if (l >= 0)// Port error not happens?
				{	// Normal or time out (Port is alive)
					if (SendCmd(0x0e, 0, 0xf0f0))// Send final packet ACK
						return -1;// Port error return
				}
			}
			if(abort)	// If user abort request?
				progressBar1.Value = 0;// Clear progress bar
			label3.Text = "";	// Clear display text
			return 0;	// Normal return
		}

		//
		//	Button enable control
		//
		private void EnableButton(bool sw)
		{
			if (sw)	// if true
			{
				button1.Enabled = true;	// Enable take picture button
				button2.Enabled = true;	// Enable take continious picture button
				button3.Enabled = true;	// Enable record button
				button4.Enabled = false;// Disable stop button
				button5.Enabled = true;	// Enable folder button
				button6.Enabled = true;	// Enable playback button
				label7.Text = "";				// Clear timer counter text
				progressBar1.Value = 0;	// Clear progress bar
			}
			else
			{
				button1.Enabled = false;// Disable take picture button
				button2.Enabled = false;// Disable take continious picture button
				button3.Enabled = false;// Disable record button
				button4.Enabled = true;	// Enable stop button
				button5.Enabled = false;// Disable folder button
				button6.Enabled = false;// Disable playback button
			}
		}

		//
		//	Save JPEG file
		//
		private void SaveFile()
		{
			int seq = 10000+ saveseq;		// Make file name seq number
			string tail = seq.ToString();// Get number string
			string fname = filename + tail.Substring(1,4)+".jpg";// Get save filename
		
			FileStream fs = new FileStream(fname, FileMode.Create ,FileAccess.Write);// Create new file
			int fileSize = (int)ms.Length;// Get memopry stream length
			byte[] buf = new byte[1024];// Data copy buffer
			int readSize;								// Read size
			int remain = fileSize;			// Remain size init value is memory stream size
			ms.Seek(0,SeekOrigin.Begin);// Rewind memory stream

			while (remain > 0)					// Data remains?
			{
				readSize = ms.Read(buf, 0, Math.Min(1024, remain));// Read memory stream
				fs.Write(buf, 0, readSize);// Write to file
				remain -= readSize;				// Subtract remain size
			}
			fs.Dispose();								// Close wrote file
			label8.Text = tail.Substring(1);	// Show current saving file seq number
		}


		//
		//	Load file
		//
		private bool LoadFile()
		{
			int seq = 10000 + playseq;// Make filename seq number
			int final;			// End seq number
			string tail = seq.ToString();// Get seq number string
			string fname = filename + tail.Substring(1, 4) + ".jpg";// Make read filename
			string str;			// Read string buffer
			StreamReader sr;// Stream reader for text access
			FileStream fs;	// File stream for read
			MemoryStream pms;// Memory stream for write and display

			try
			{
				sr = new StreamReader(filename + ".seq");// Try to read final seq number file
				str = sr.ReadLine();// Get one line
				sr.Close();		// Close after one line read
				try
				{
					final = Int32.Parse(str);// Get final seq number from string
				}
				catch
				{
					final = 0;	// Clear final seq number when error happens
				}
				if (playseq >= final)// If reqested seq number exceed wrote file seq number?
					return true;// Error return (abort playback)
			}
			catch	// Index file not found
			{
			}
			try
			{
				fs = new FileStream(fname, FileMode.Open, FileAccess.Read);// Try to open file
			}
			catch
			{
				return true;	// Not found then error return (abort playback)
			}
			System.IO.FileInfo fi = new System.IO.FileInfo(fname);// Get file info to get time stamp
			label3.Text = fi.LastWriteTime.ToLongDateString() + " " + fi.LastWriteTime.ToLongTimeString();// Show date & time
			label3.Update();// Update date & time
			int fileSize = (int)fs.Length;// Get JPEG file size
			byte[] buf = new byte[1024];// JPEG data buffer
			int readSize;		// Read counter
			int remain = fileSize;// Total length of JPEG data file

			pms = new MemoryStream(fileSize);// Create memory stream for display

			while (remain > 0)	// JPEG file data remains?
			{
				readSize = fs.Read(buf, 0, Math.Min(1024, remain));// Read data from file
				pms.Write(buf, 0, readSize);// Copy to memory stream
				remain -= readSize;// Subtract JPEG file total length
			}
			fs.Dispose();				// Close file after read
			try
			{
				pictureBox1.Image = new Bitmap(pms);// Display JPEG memory stream
				pictureBox1.Update();// Update picture box
			}
			catch
			{
			}
			pms.Dispose();// Dispose memory stream after display
			label8.Text = tail.Substring(1);	// Show current seq numver
			return false;	// Normal return
		}

		//
		//Abort action
		//
		private void AbortCmd()
		{
			EnableButton(true);// Enable all button execpt stop button
			timer1.Enabled = false;// Disable timer
			timercount = 0;
			cmd = 0;		// Clear action command
		}

		//
		//	Background action thread
		//
		private void backgroundWorker1_DoWork(object sender, DoWorkEventArgs e)
		{
			int i = 0;	// Return value temp work
			StreamWriter sr;// Steam for write JPEG file
			active = true;// Thread active flag
			for (; ; )
			{
				if (quit)	// User abort request?
					break;	// Exit action thread loop
				switch(cmd)// Execute each action
				{
				case 1:		// Take single picuture command?
				case 2:		// Take continuous picture command?
				case 3:		// Record command?
					if(cmd == 3 && !checkBox4.Checked)// Only record command and not stop motion mode
					{
						if (abort)// User abort request?
						{
							AbortCmd();// Clear action command and go idle
							break;// break switch {}
						}
						if (timercount > 0)// Interval timer set?
							break;// break switch {} (wait untill timercount == 0)
						try
						{
							timercount = Int32.Parse(textBox1.Text);// Get next interval count from text box
						}
						catch
						{
							timercount = 0;// interval time is 0
						}
					}
						
					abort = false;// Clear user abort flag
                    label3.Text = "";	// No probrem then show blank text
                    i = GetPict();// Get JPEG data from camera
                    if (i != 0)		// Some error happens?
					{
						if (i > 0)	// Timeout or sequence error
						{
							if (cmd == 2 || checkBox4.Checked)	// Continue mode?
							{
								label3.Text = label3.Text + " 20bJɃANZXȂŉ";// Show error
								AbortCmd();// Abort continious command when error(go idle)
							}
							else if (cmd == 3 && Int32.Parse(textBox1.Text) < 20)// Record mode and interval < 20 sec?
							{
								label3.Text = label3.Text + " I[gp[It҂";// Show status
								timercount = 20;	// Force set wait 20 sec to wait camera shutdown
							}
                            else
                                label3.Text = "J^CAEg";// Show status
                        }
						else if(i < 0)// Port error happens?
						{
							try
							{
								serialPort1.Close();// Try to close serial port anyway
							}
							catch	// Fail to close serial port
							{
							}
							AbortCmd();	// Abort current action command and go idle state
						}
						break;	// continue if cmd == 3 else stop
					}
					label3.Text = "";	// No probrem then show blank text
					if (abort)	// User abort request has come?
					{
						AbortCmd();// Abort current action by user abort
						break;		// Exit switch and go idle state
					}
					try
					{
						pictureBox1.Image = new Bitmap(ms);// Show JPEG data
						pictureBox1.Update();// Update picture box
					}
					catch	// ms is broken and can't display
					{
					}
					if(cmd == 3)	// Record mode?
					{
						SaveFile();	// Save memory stream
						saveseq++;	// Advance JPEG file name seq number
						sr = new StreamWriter(filename+".seq");// Create final index file name
						sr.Write(saveseq.ToString());// Write final record seq number
						sr.Close();	// Close final index file just after write seq number
						if (checkBox4.Checked)	// Stop motion mode?
							AbortCmd();	// Abort makes one shot recording
					}
					break;				// Exit switch {}

				case 4:				// Playback action
					if (abort || LoadFile())// User avort request or fail to read JPEG file?
					{
						AbortCmd();// Abort playback and go idle state
						break;		// Exit switch {}
					}
					playseq++;	// Advance playback seq number for next JPEG file read
					break;			// Exit switch {}

				}
				if (cmd == 1)	// Take single picture command?
					AbortCmd();	// Abort single picture command
				System.Threading.Thread.Sleep(1);// Allow another thread work
			}
			active = false;	// User abort and exit for(;;) loop here
		}

		//
		//Take single picture button
		//
		private void button1_Click(object sender, EventArgs e)
		{
			label8.Text = "";
			cmd = 1;		// Set take single picture command
			EnableButton(false);// Setup buttons
		}

		//
		// Take continious picture button
		//
		private void button2_Click(object sender, EventArgs e)
		{
			label8.Text = "";
			cmd = 2;		// Set take continious picture command
			EnableButton(false);// Setup buttons
		}

		//
		//Record button
		//
		private void button3_Click(object sender, EventArgs e)
		{
			string str;	// Seq num string work
			if(filename == "")// Save file folder is not set yet?
			{
				MessageBox.Show("摜Z[utH_ƃt@Cw肳Ă܂", "L^");// Show error dialog
				return;		// Do nothing
			}
			try
			{
				StreamReader sr = new StreamReader(filename+".seq");// Open final seq number file
				str = sr.ReadLine();// Get one line
				sr.Close();// Close file just after read one line
			}
			catch				// Not found?
			{
				str = "0";// Set str is '0'
			}

			try
			{
				saveseq = Int32.Parse(str);// Get number from read strind
			}
			catch		// Can't parse?
			{
				saveseq = 0;// Force set 0
			}
			abort = false;// Clear user abort flag
			timer1.Enabled = true;// Activate timer
			cmd = 3;			// Set record command
			EnableButton(false);// Setup buttons
		}

		//
		// Stop button
		//
		private void button4_Click(object sender, EventArgs e)
		{
			abort = true;	// Set user abort flag for background thread
			AbortCmd();		// Abort and setup buttons
		}

		//
		// Folder button
		//
		private void button5_Click(object sender, EventArgs e)
		{
			DialogResult ret;	// FileOpenDialog result work

			saveFileDialog1.Title = "摜t@C̎w";	// Set dialog box title
			saveFileDialog1.CheckFileExists = false;		// Disable exit check
			saveFileDialog1.RestoreDirectory = true;		// Save current folder flag set
			saveFileDialog1.Filter = "seqt@C(*.seq)|*.seq";// File file filter

			ret = saveFileDialog1.ShowDialog();	// Show file open dialog box
			if (ret == System.Windows.Forms.DialogResult.OK)//OK button pressed?
			{
				filename = saveFileDialog1.FileName;	// get file path string
				if (filename.Substring(filename.Length - 4, 4) == ".seq")// Tail includes ".seq"?
					filename = filename.Substring(0, filename.Length - 4);// Cut tail ".seq"
			}
		}

		//
		// Playback button
		//
		private void button6_Click(object sender, EventArgs e)
		{
			if (filename == "")// Folder not set yet?
			{
				MessageBox.Show("Đ摜̃tH_ƃt@Cw肳Ă܂", "Đ");// Show error
				return;		// Do nothing
			}
			playseq = 0;// Clear playback seq number
			label8.Text = "";	// Clear Playback seq number
			abort = false;// Clear user abort flag
			cmd = 4;		// Set playback command
			EnableButton(false);// Setup buttons
		}

		//
		// Port name combo box
		//
		private void comboBox1_SelectedIndexChanged(object sender, EventArgs e)
		{
            if (comboBox3.SelectedIndex >= 0)// Index is valid?
			{
                label3.Text = "";	// No probrem then show blank text
                port = comboBox1.Text;// get selected port name
				try
				{
					serialPort1.Close();// Close privious opened port
				}
				catch	// No port is attached yet
				{
				}
                serialPort1.PortName = port;	// Set port name
                try
                {
                    serialPort1.Open();	// Try to open serial port
                    serialPort1.RtsEnable = true;// Enable RTS for flow control
                }
                catch (Exception ex)	// Open error happens?
                {
                    MessageOut(ex.Message);
                    return;				// Error return
                }
            }

		}

		//
		// Resolution combo box
		//
		private void comboBox2_SelectedIndexChanged(object sender, EventArgs e)
		{
			if (comboBox2.SelectedIndex >= 0)// Valid index?
				sizeindex = comboBox2.SelectedIndex;// set sizeindex
		}

		//
		// Baud rate combo box
		//
		private void comboBox3_SelectedIndexChanged(object sender, EventArgs e)
		{
			if (comboBox3.SelectedIndex >= 0)	// Valid index?
				baudindex = comboBox3.SelectedIndex;// Set baudindex
		}

		//
		// Light frequency combo box
		//
		private void comboBox4_SelectedIndexChanged(object sender, EventArgs e)
		{
			if (comboBox4.SelectedIndex >= 0)// Valid index?
				freq = comboBox4.SelectedIndex;// Set freq work
		}

		//
		// Picture box click handler
		//
		private void pictureBox1_Click(object sender, EventArgs e)
		{
			if (++boxsize == 5)// Size wrap around?
				boxsize = 0;	// Back to full size
			if (boxsize < 4)
			{
				pictureBox1.Location = picpos;	// Set normal picture box position
			}
			else
			{
				pictureBox1.Location = new Point(0, 0);	// Set full screen
			}
			switch (boxsize)// Branch for each size
			{
			case 0:	// Full size
				pictureBox1.Width = this.Width - margine;// Get full size width
				pictureBox1.Height = pictureBox1.Width * 3 / 4;// Get full size hight
				break;
			case 1:	// 320x240
				pictureBox1.Width = 320;
				pictureBox1.Height = 240;
				break;
			case 2:	// 160x120
				pictureBox1.Width = 160;
				pictureBox1.Height = 120;
				break;
			case 3:	// 80x64
				pictureBox1.Width = 80;
				pictureBox1.Height = 64;
				break;
			case 4:	// Full screen
				pictureBox1.Width = this.ClientSize.Width;
				pictureBox1.Height = this.ClientSize.Width * 3 / 4;
				break;
			}
			pictureBox1.Update();	// Update new size picture box
		}

		//
		// Timer handler
		//
		private void timer1_Tick(object sender, EventArgs e)
		{
			if (timercount > 0)// if Timer has value
				timercount--;		// Decrement timer value
			if(!checkBox4.Checked)
				label7.Text = timercount.ToString();// Show interval timer value
			label7.Update();	// Update text
		}

		private void checkBox2_CheckedChanged(object sender, EventArgs e)
		{
			if(serialPort1.IsOpen)					// Serial port open now?
				serialPort1.RtsEnable = true;	// Force set RTS on
		}

		private void checkBox4_CheckedChanged(object sender, EventArgs e)
		{
			textBox1.Enabled = !checkBox4.Checked;	// If stop motion then disable interval time
			if (checkBox4.Checked)			// Stop motion takes time
				checkBox3.Checked = true;	// AGC setteing is nessary
		}

        private void checkBox5_CheckedChanged(object sender, EventArgs e)
        {
            checkBox3.Enabled = !checkBox5.Checked;
        }

        private void comboBox5_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (comboBox5.SelectedIndex == 1)    // Wireless mode?
            {
                checkBox2.Enabled = true;// Set light check box enable
                comboBox3.SelectedIndex = 3;	// Force set 57600 bps
            }
            else if (comboBox5.SelectedIndex == 1)    // Wireless mode?
            {
                checkBox2.Enabled = false;// Set light check box enable
                comboBox3.SelectedIndex = 2;	// Force set 57600 bps
            }
            else
                checkBox2.Enabled = false;// Set light check box enable
        }

        private void checkBox1_CheckedChanged(object sender, EventArgs e)
        {

        }

	}
}
