//
//	Prototypes for all
//
typedef void(*CallbackFunc16)(UINT16);
typedef void(*CallbackFunc8)(UINT8);
typedef void(*CallbackFuncV)(void);

//
//	Constants
//

#define	SEEK_SET	0
#define	SEEK_CUR	1
#define	SEEK_END	2

//
//	Macros
//

//
//	Kernel
//
void		SystemSleep(void);
void		KernelInit(void);
UINT8		CreateThread(UINT16 adrs, UINT16 stacksize, UINT16 arg);
void		RtiEnable(void);
void		RtiDisable(void);
void		*MemoryAlloc(UINT16 size);
void		*MemoryAllocSub(UINT16 size);	// Don't touch interrupt flag (Kernel Internal)
void		MemoryFree(void *);
void		MemoryFreeSub(void *);				// Don't touch interrupt flag (Kernel internal)
UINT8		CreateTimer(UINT8 mode, CallbackFunc8 func, UINT16 val, UINT8 timer, UINT8 stack);
UINT16	GetTimerCount(UINT8 timer);
void		Sleep(UINT16 time);
void		StartAllThread(void);
void		WaitCriticalSession(UINT8 *flag);
void		KillThread(UINT8 handle);
//
//	FileSys
//
void		FormatFilesys(UINT16 key);
void		DefragFilesys(UINT16 key);
void		FixFileStruct(UINT32 pos);
UINT32	GetFinalPos(void);

UINT8		CreateFile(INT8 *filename);
UINT16	WriteFile(void *ptr, UINT16 size);
void		CloseFile(UINT16 handle);
UINT8		DeleteFile(INT8 *filename);
UINT8		MoveFile(INT8 *filename, INT8 *newfile);
INT8		*FindFile(UINT8 init);
UINT32	GetFileSize(INT8 *filename);
UINT32	GetFinalPos(void);
UINT16	OpenFile(INT8 *filename);
UINT16	ReadFile(UINT16 handle, UINT8 *buf, UINT16 size);
UINT32	SeekFile(UINT16 fp, UINT32 pos, UINT8 org);

//
//	Bios
//
void		BufCopy(void *dest, void *src, UINT16 size);
void		MemClear(void *dest, UINT16 size);
UINT8		NvmErase(UINT16 *adrs);
UINT8		NvmWrite(UINT16 *ad, UINT16 *src, UINT16 size);
void		WriteSrecord(INT8 *Buf);

void		DebugPutc(UINT8 c);
void		DebugPuts(void *s);
void		DebugPutHex(UINT8 c);
void		DebugPutHex2(UINT16 c);
void		DebugPutNum(INT32 c);
void		GetDigit(INT32 num, UINT8 *buf);

UINT8		SciSense(UINT8 ch);
UINT8		SciGetc(UINT8 ch);
void		SciPutc(UINT8 c, UINT8 ch);
void		SciPuts(INT8 *str, UINT8 ch);
void		SciInit(void);
void		SetSciVector(UINT8 vecnum, void *adrs);

void		SaveSafeMode(UINT16 key);

//
//	Flash block
//
UINT8		FlashGetStatus(void);
UINT8		FlashRead(UINT16 bank, UINT8 offset);
void		FlashBlockRead(UINT16 bank, UINT8 offset, UINT8 *buffer, UINT16 size);
void		FlashWait(void);
void		FlashWrite(UINT16 bank, UINT8 offset, UINT8 *buffer, UINT16 size);
void		FlashEraseWait(UINT8 bank);

//
//	Ether
//
void		EtherInit(UINT8 enable);
void		EtherSend(UINT8 *buf, UINT16 len);

void		CopySixBytes(UINT8 *dst, UINT8 *src);
void		CopyMyMAC(UINT8 *dst);
void		CopyFourBytes(UINT8 *dst, UINT8 *src);

void		EtherFreeBuffer(EtherBuf *etherptr);
void		EtherHandler(void);
void		EmacEnable(UINT8 enable);
void		DispatchPacket(EtherBuf *etherptr);

//
//	Arp
//
void		ArpInit(void);
void		ArpHandler(EtherBuf *);
ArpTable* ArpFind(UINT32 ip);
void		ArpRefresh(void);
INT8		AddArpCache(UINT32 ip, UINT8* mac, UINT8 type);

//
//	IP
//
void		IpInit(void);
UINT16	IpHandler(EtherBuf *etherptr);
UINT16	IpCheckSumBlock(UINT16 sum, UINT8 *buf, UINT16 len);
UINT8		IpPacketSend(UINT32 ip, UINT8 prot, UINT8 *hdr, UINT8 hlen, UINT8 *buf, UINT16 len);

//
//	ICMP
//
void		SendIcmpPacket(UINT8 *buf, UINT16 len, UINT32 destip, UINT16 prot);
void		IcmpHandler(EtherBuf *etherptr);

//
//	UDP
//
void		SocketInit(void);

void		UdpHandler(EtherBuf *etherptr);
UINT16	GetFreePort(void);

INT8		CreateSocket(UINT8 prot);
UINT8		Bind(INT8 sochandle, UINT16 port, UINT8 maxsoc);
void		CloseSocket(INT8 sochandle);
UINT8		CheckSocketActive(INT8 sochandle);

UINT32	GetSenderIP(INT8 sochandle);
UINT16	GetSenderPort(INT8 sochandle);

UINT8		*GetReceiveBuffer(INT8 sochandle, UINT8 release);

INT16		SendTo(INT8 sochandle, UINT32 ip, UINT16 port, UINT8 *buf, UINT16 len);
INT16		RecvFrom(INT8 sochandle, UINT16 timeout);

//
//	TCP
//
void		TcpHandler(EtherBuf *etherptr);
void		TcpState(void);

INT16		GetDefMtu(INT8 sochandle, UINT16 mtu);
INT8		Connect(INT8 sochandle, UINT32 ip, UINT16 port);
INT8		Accept(INT8 sochandle, UINT16 timeout);
UINT32	GetRandomSeq(void);

INT16		Read(INT8 sochandle, UINT16 timeout);
INT16		Write(INT8 sochandle, UINT8 *buf, UINT16 len);
INT8		WaitWriteComplete(INT8 sochandle);
INT8		CheckWriteComplete(INT8 sochandle);

void		CloseTcpSocket(socket *soc);

//
//	TFTP server
//
void		TftpInit(void);

//
//	HTTP server
//
void		HttpInit(void);

//
//	Resolve
//
UINT32 GetHostByName(UINT8 *name);

//
//	Registry
//
void LoadRegistry(VarEntry* RegTable);
INT8 SaveRegistry(VarEntry* RegTable);
void UpdateString(VarEntry* RegTable);
UINT8 GetHexTwo(UINT8 *ptr);
UINT32 *GetIpAdrs(void);
//
//	CGI
//

UINT8 *ReplaceCgi(UINT8 *buf, UINT16 size);
void DoCgi(UINT8 *ptr);
UINT16	ReadFileCgi(UINT16 handle, UINT8 *buf, UINT16 size, UINT8 *extra);

//
//	DHCP
//
void DhcpInit(UINT8 server);
//
//	Main
//
void SetUserProgram(UINT8 *buf, UINT16 adrs);

